
local mod = ...


mod:registerBehaviorTreeNode({
    Name = "BEHAVIOUR_NODE_GUILD_LEVY_WORK",
    VariableList = {
		Var1 = "BEHAVIOR_TREE_DATA_AGENT",
		Var2 = "BEHAVIOR_TREE_DATA_WAIT",
		Var3 = "BEHAVIOR_TREE_DATA_LOCATION",
		Var4 = "BEHAVIOR_TREE_DATA_LOCATION",
		Var5 = "BEHAVIOR_TREE_DATA_LOOP",
		Var6 = "BEHAVIOR_TREE_DATA_LOOP",
		Var7 = "BEHAVIOR_TREE_DATA_LOOP",
		Var8 = "BEHAVIOR_TREE_DATA_WAIT",
    },

    Init = function(self, instance)

        return BEHAVIOR_TREE_NODE_RESULT.TRUE

    end,
    Update = function(self, instance)

        return BEHAVIOR_TREE_NODE_RESULT.TRUE

    end,
    
    Finish = function(self, instance)

        return BEHAVIOR_TREE_NODE_RESULT.TRUE

    end,

})



-- Node: Move to next available Levy --------------------------------------------------------------------------------------------------------
mod:registerBehaviorTreeNode({
    Name = "BEHAVIOUR_NODE_GUILD_LEVY_MOVE",
    VariableList = {
		AgentData = "BEHAVIOR_TREE_DATA_AGENT",
        MovePosition = "BEHAVIOR_TREE_DATA_LOCATION",
    },

    Update = function(self, instance)

        local compAgent = nil
        compAgent = self.AgentData.Agent

        local posAgent = nil
        posAgent = compAgent:getOwner():getGlobalPosition()

        local moveSpot = nil
        moveSpot = self.MovePosition

        local dist = nil
        local posTraining = nil
        local objGuildWorkplace = nil
        local compGuildWorkplace = nil

        local listTraining = nil
        listTraining = {}

        -- Find the worker's job
        local job = nil
        job = compAgent:getOwner():getComponent("COMP_VILLAGER"):getAssetJob()
        local jobName = nil

        -- mod:log("Starting Guild Levy Collector movement node")

        if job then

            -- Find out what type of Guild this member belongs to
            local guildType = nil
            jobName = job.JobName
            guildType = string.sub(jobName, 16, string.len(jobName) - 5)

            -- mod:log(guildType)

            if guildType then

                -- Find all guild trainings for this guild
                local compTraining = nil
                compAgent:getLevel():getComponentManager("COMP_GUILD_TRAINING_" .. guildType):getAllEnabledComponent():forEach(function(compTraining)

                    compGuildWorkplace = compTraining:getOwner():getComponent("COMP_GUILD_WORKPLACE")

                    if compGuildWorkplace then

                        -- mod:log("Found comp Training matching this guild type. Timer = " .. tostring(compGuildWorkplace.TimerLevy))

                        -- Only register this training if it has not been used recently
                        if compGuildWorkplace.TimerLevy == 0 then

                            posTraining = compTraining:getOwner():getGlobalPosition()
                            dist = math.sqrt( (posAgent.x - posTraining.x)^2 + (posAgent.z - posTraining.z)^2 )
                            table.insert(listTraining, {compGuildWorkplace = compGuildWorkplace, dist = dist})

                        end

                    end

                end)

                -- Sort the trainings by distance
                table.sort(listTraining, SortByDist)

                -- Set destination as the nearest one
                -- Set destination as the nearest one
                if #listTraining > 0 then

                    compGuildWorkplace = listTraining[1].compGuildWorkplace
                    if compGuildWorkplace then

                        -- Find the entrance to a worker, pickup or default path for the target building
                        objGuildWorkplace = compGuildWorkplace:getOwner()
                        local destPath = nil
                        local testObj = nil
                        local compPart = objGuildWorkplace:getComponent("COMP_BUILDING_PART")
                        if compPart then
                            for i = 1, #compPart.PathList do
                                if not destPath then
                                    if compPart.PathList[i].PathType ~= BUILDING_PATH_TYPE.LIVESTOCK then
                                        testObj = compPart.PathList[i].WayPointList[1]
                                        if testObj then
                                            destPath = testObj
                                        end
                                    end
                                end
                            end
                        end

                        if destPath then
                            moveSpot:setDestination(destPath)
                        end

                        compGuildWorkplace.TimerLevy = 14
                        return BEHAVIOR_TREE_NODE_RESULT.TRUE

                    end

                end

            end

        end

        return BEHAVIOR_TREE_NODE_RESULT.FALSE

    end -- update

})



-- Node: Collect Tax -------------------------------------------------------------------------------------
mod:registerBehaviorTreeNode({
    Name = "BEHAVIOUR_NODE_GUILD_LEVY_COLLECT",
    VariableList = {
		AgentData = "BEHAVIOR_TREE_DATA_AGENT",
    },

    Update = function(self, instance)

        local compAgent = nil
        compAgent = self.AgentData.Agent

        local compTaxOffice = nil
        compTaxOffice = compAgent:getOwner():getComponent("COMP_VILLAGER"):getJobInstance().Workplace:getOwner():getComponent("COMP_WORKPLACE_TAX_OFFICE")

        if compTaxOffice then

            compTaxOffice:depositTaxesCollected("GOLD_COINS", 20)

        end

        return BEHAVIOR_TREE_NODE_RESULT.TRUE

    end

})



-- **********************************************************************************************************************************************
-- ***************** T R E E ********************************************************************************************************************
-- **********************************************************************************************************************************************
mod:registerBehaviorTree({
    Id = "BEHAVIOR_GUILD_LEVY",
    VariableList = {
        {
            Name = "AgentData",
            DataType = "BEHAVIOR_TREE_DATA_AGENT",
            IsPublic = false,
            InitialValue = {}
        },
		{
			Name = "DoJobTimer",
			DataType = "BEHAVIOR_TREE_DATA_WAIT",
			IsPublic = false,
			InitialValue = {
				TimeToWait = 0,
				Animation = AGENT_ANIMATION.IDLE,
				SetIdleAfterWait = true,
			}
		},
		{
			Name = "WorkplacePosition",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION",
			IsPublic = false,
			InitialValue = {
				CanNavigateOnGround = true,
				CanNavigateOnWater = false,
				IsSetOrientationOnDestination = true,
			}
		},
        {
            Name = "MovePosition",
            DataType = "BEHAVIOR_TREE_DATA_LOCATION",
            IsPublic = false,
            InitialValue = {
                CanNavigateOnGround = true,
                CanNavigateOnWater = false,
                GotoObjectPosition = true,
                IsSetOrientationOnDestination = true
            }
        },
		{
			Name = "WorkLoop",
			DataType = "BEHAVIOR_TREE_DATA_LOOP",
			IsPublic = true,
			InitialValue = {
				LoopCount = 1,
				Duration = 0,
				IsInfinite = false,
				IsDuration = false,
			}
		},
		{
			Name = "LevyLoop",
			DataType = "BEHAVIOR_TREE_DATA_LOOP",
			IsPublic = true,
			InitialValue = {
				LoopCount = 0,
				Duration = 120, -- will do one work cycle over two days, after that will check if needs must be met and quit if so
				IsInfinite = false,
				IsDuration = true,
			}
		},
		{
			Name = "ResourceProducedList",
			DataType = "BEHAVIOR_TREE_DATA_RESOURCE_PRODUCTION",
			IsPublic = false,
		},
    },
	Root = {
		Name = "WorkingGlobalSequencer",
		Type = "SEQUENCER",
		Children = {
			{
				Name = "LevyWorkLoop",
				Type = "BEHAVIOUR_NODE_GUILD_LEVY_WORK",
				Var1 = "AgentData",
				Var2 = "DoJobTimer",
				Var3 = "WorkplacePosition",
				Var4 = "MovePosition",
				Var5 = "WorkLoop",
				Var6 = "LevyLoop",
			},
			{
				Name = "WorkLoopRepeater",
				Type = "REPEAT",
				RepeatData = "WorkLoop",
				Child = {
					Name = "WorkLoopSequencer",
					Type = "SEQUENCER",
					Children = {
						{
							Name = "SetupWork",
							Type = "SETUP_WORK",
							AgentData = "AgentData",
							TimeToWait = "DoJobTimer",
							WorkPosition = "WorkplacePosition",
						},
                        {
							Name = "GoToWorkStart",
							Type = "GO_TO",
							AgentData = "AgentData",
							Destination = "WorkplacePosition",
							BuildingPathType = "",
							AnimationData = "",
							AnimationSpeedMultiplier = "",
						},
                        {
                            Name = "LevyLoopRepeater",
                            Type = "REPEAT",
                            RepeatData = "LevyLoop",
                            Child = {
                                Name = "LevyLoopSequencer",
                                Type = "SEQUENCER",
                                Children = {
                                    {
                                        Name = "SetNextLevy",
                                        Type = "BEHAVIOUR_NODE_GUILD_LEVY_MOVE",
                                        AgentData = "AgentData",
                                        MovePosition = "MovePosition",
                                    },
                                    {
                                        Name = "GoToLevy",
                                        Type = "GO_TO",
                                        AgentData = "AgentData",
                                        Destination = "MovePosition",
                                        BuildingPathType = "",
                                        AnimationData = "",
                                        AnimationSpeedMultiplier = "",
                                    },
                                    {
                                        Name = "ProduceResource",
                                        Type = "PRODUCE_RESOURCE",
                                        AgentData = "AgentData",
                                        TimeToWait = "DoJobTimer",
                                        ResourceProducedList = "ResourceProducedList",
                                    },
                                    {
                                        Name = "CollectLevy",
                                        Type = "BEHAVIOUR_NODE_GUILD_LEVY_COLLECT",
                                        AgentData = "AgentData",
                                    },
                                },
                            },
                        },
                        {
							Name = "GoToWorkFinish",
							Type = "GO_TO",
							AgentData = "AgentData",
							Destination = "WorkplacePosition",
							BuildingPathType = "",
							AnimationData = "",
							AnimationSpeedMultiplier = "",
						},
                        {
							Name = "GettingXp",
							Type = "GIVE_JOB_XP",
							AgentData = "AgentData",
							ShouldReceiveXp = "",
						}
					}
				}
			}
		}
	}
})

